--
-- Automatic Belt Direction
-- A Factorio mod
-- Author: Magic Power
--


-- The belts layed after each other are stored in here.
previousBelt = nil
 currentBelt = nil

-- Remember the direction the belts were originally pointing when placed. This helps detect in-hand belt turning.
previousOriginalDirection = nil
 currentOriginalDirection = nil


-- Function IsAdjacent()
--
-- Checks whether two positions are adjacent, and return which side the latter is.
--
-- Takes two {x,y} Position tables through pos1 and pos2.
-- Returns if pos2 is adjacent to pos1, horizontally or vertically.
-- The result returned is the direction pos2 is to pos1, or nil if they are not adjacent.
-- For ease of use, the direction returned is the same as the belt direction constants.
--
local function IsAdjacent(pos1,pos2)
  local differenceinX = pos2.x - pos1.x
  local differenceinY = pos2.y - pos1.y

  if differenceinX == 0 then
    if differenceinY == -1 then return defines.direction.north end
    if differenceinY == 1  then return defines.direction.south end
  end
  if differenceinY == 0 then
    if differenceinX == -1 then return defines.direction.west end
    if differenceinX == 1  then return defines.direction.east end
  end

  return nil
  
end -- IsAdjacent()




-- Function ChangeBelts()
--
-- When building transport belts, automatically change the direction of the belts
-- if the belts are placed adjacent.
--
-- Takes the current item built through 'event'.
-- Changes the belt direction if needed.
-- Returns true if a valid belt was found, false otherwise.
--
function ChangeBelts()
  -- Only handle belts placed.
  if currentBelt.type ~= 'transport-belt' then return false end

  -- The previous item had to be a belt.
  if not previousBelt then return true end

  -- The belts have to be adjacent.
  -- Also, this directly determines what direction the belts are being layed.
  local beltDirection = IsAdjacent(previousBelt.position, currentBelt.position)
  if beltDirection == nil then return true end

  -- When a player turns the new belt before placing it, then do not change belt directions.
  -- (Since the player obviously has other plans for the belts).
  if previousOriginalDirection ~= currentOriginalDirection then return true end
  
  -- Change both belts direction.
  previousBelt.direction = beltDirection
   currentBelt.direction = beltDirection
  
  return true

end -- ChangeBelts()




-- Event: on_built_entity
--
-- When building transport belts, automatically change the direction of the belts
--
script.on_event(defines.events.on_built_entity, function(event)

  currentBelt = event.created_entity
  currentOriginalDirection = currentBelt.direction
  
  -- Main task handled by subfunction, to better handle exit conditions.
  -- ChangeBelts returns if a belt was found. Only belts are remembered.
  if ChangeBelts() == true then
    previousBelt = currentBelt
    previousOriginalDirection = currentOriginalDirection
  else
    previousBelt = nil
    previousOriginalDirection = nil
  end

end)




--
-- Catch events that (may) remove the previous belt remembered in this script,
-- or rotate/alter the previous belt that changes how we should handle it.
-- If they occur, remove any previous belt.
--
script.on_event(defines.events.on_player_mined_entity, function(event)
  previousBelt = nil
end)

script.on_event(defines.events.on_player_rotated_entity, function(event)
  previousBelt = nil
end)

script.on_event(defines.events.on_robot_mined_entity, function(event)
  previousBelt = nil
end)
